@@ -86,7 +86,7 @@ void CardinalityStore<ITEM, TAG>::applyChange(const PatchList<ITEM, TAG>& change
 template<class ITEM, class TAG>
 void CardinalityStore<ITEM, TAG>::TagContainer::add(const TAG& tag, int card) throw ()
 {
-	typename TagContainer::iterator i = find(tag);
+	typename TagContainer::iterator i = this->find(tag);
 	if (i == TagContainer::end())
 		insert(make_pair(tag, card));
 	else
@@ -96,7 +96,7 @@ void CardinalityStore<ITEM, TAG>::TagContainer::add(const TAG& tag, int card) th
 template<class ITEM, class TAG>
 void CardinalityStore<ITEM, TAG>::TagContainer::del(const TAG& tag, int card) throw ()
 {
-	typename TagContainer::iterator i = find(tag);
+	typename TagContainer::iterator i = this->find(tag);
 	if (i != TagContainer::end())
 		if (i->second > card)
 			i->second -= card;
@@ -692,7 +692,16 @@ void to::test<1>()
 {
 	CardinalityStore<string, string> coll;
 
-	test_tagged_collection(coll);
+	output_test_collection(coll);
+	test_readonly_collection(coll);
+}
+
+template<> template<>
+void to::test<2>()
+{
+	CardinalityStore<string, string> coll;
+
+	test_collection(coll);
 }
 
 }
