@@ -19,6 +19,7 @@
  */
 
 #include <tagcoll/TDBDiskIndex.h>
+#include <tagcoll/Patches.h>
 
 #include <tdb.h>
 #include <fcntl.h>	// O_RDONLY
@@ -38,10 +39,10 @@ template<class ITEM, class TAG>
 TDBDiskIndex<ITEM, TAG>::TDBDiskIndex(
 		const std::string& pkgidx,
 		const std::string& tagidx,
-		Converter<ITEM, std::string>& fromitem,
-		Converter<TAG, std::string>& fromtag,
-		Converter<std::string, ITEM>& toitem,
-		Converter<std::string, TAG>& totag,
+		const Converter<ITEM, std::string>& fromitem,
+		const Converter<TAG, std::string>& fromtag,
+		const Converter<std::string, ITEM>& toitem,
+		const Converter<std::string, TAG>& totag,
 		bool write) :
 			pkgdb(pkgidx), tagdb(tagidx),
 			fromitem(fromitem), fromtag(fromtag),
@@ -79,6 +80,7 @@ OpSet<TAG> TDBDiskIndex<ITEM, TAG>::getTagsOfItem(const ITEM& item) const
 }
 
 #ifndef INSTANTIATING_TEMPLATES
+namespace Tagcoll {
 template<>
 OpSet<string> TDBDiskIndex<string, string>::getItemsHavingTag(const string& tag) const
 {
@@ -90,6 +92,7 @@ OpSet<string> TDBDiskIndex<string, string>::getTagsOfItem(const string& item) co
 {
 	return pkgdb.getStringSet(item);
 }
+}
 #endif
 
 
@@ -137,13 +140,13 @@ OpSet<TAG> TDBDiskIndex<ITEM, TAG>::getAllTags() const
 template<class ITEM, class TAG>
 struct out_data
 {
-	Converter<std::string, ITEM>& toitem;
-	Converter<std::string, TAG>& totag;
+	const Converter<std::string, ITEM>& toitem;
+	const Converter<std::string, TAG>& totag;
 	Consumer<ITEM, TAG>& consumer;
 
 	out_data(
-			Converter<std::string, ITEM>& toitem,
-			Converter<std::string, TAG>& totag,
+			const Converter<std::string, ITEM>& toitem,
+			const Converter<std::string, TAG>& totag,
 			Consumer<ITEM, TAG>& consumer) throw ()
 		: toitem(toitem), totag(totag), consumer(consumer) {}
 };
@@ -255,21 +258,115 @@ namespace tut {
 using namespace tut_tagcoll;
 
 struct tagcoll_tdbdiskindex_shar {
+	~tagcoll_tdbdiskindex_shar() {
+		unlink("pkgidx.test");
+		unlink("tagidx.test");
+	}
 };
 TESTGRP(tagcoll_tdbdiskindex);
 
 template<> template<>
 void to::test<1>()
 {
-	Converter<string, string> a;
+	TrivialConverter<string, string> a;
 	TDBDiskIndex<string, string> coll("pkgidx.test", "tagidx.test", a, a, a, a);
 
-	test_tagged_collection(coll);
+	output_test_collection(coll);
+	test_readonly_collection(coll);
 
 	unlink("pkgidx.test");
 	unlink("tagidx.test");
 }
 
+template<> template<>
+void to::test<2>()
+{
+	TrivialConverter<string, string> a;
+	TDBDiskIndex<string, string> coll("pkgidx.test", "tagidx.test", a, a, a, a);
+
+	test_collection(coll);
+
+	unlink("pkgidx.test");
+	unlink("tagidx.test");
+}
+
+template<> template<>
+void to::test<3> ()
+{
+	TrivialConverter<string, string> conv;
+
+	try {
+		// An empty database should return empty sets, but not fail
+		Tagcoll::TDBDiskIndex<string, string> tfi(
+				"pkgidx.test", "tagidx.test",
+				conv, conv, conv, conv);
+		gen_ensure(!tfi.hasTag("cippo"));
+		gen_ensure(!tfi.hasTag("lippo"));
+		gen_ensure(tfi.getItems("cippo") == Tagcoll::OpSet<string>());
+		gen_ensure(tfi.getTags("cippo") == Tagcoll::OpSet<string>());
+		gen_ensure(tfi.getItems("lippo") == Tagcoll::OpSet<string>());
+		gen_ensure(tfi.getTags("lippo") == Tagcoll::OpSet<string>());
+		// No autovivification should happen on gets
+		gen_ensure(!tfi.hasTag("cippo"));
+		gen_ensure(!tfi.hasTag("lippo"));
+
+		// Adding an item with no tags should still return an empty set for
+		// that item 
+		tfi.consume("cippo");
+		gen_ensure(!tfi.hasTag("cippo"));
+		gen_ensure(tfi.getItems("cippo") == Tagcoll::OpSet<string>());
+		gen_ensure(tfi.getTags("cippo") == Tagcoll::OpSet<string>());
+
+		// Add some data and check that it comes back
+		Tagcoll::OpSet<string> s;
+		s += "lippo";
+		s += "zippo";
+		s += "rippo";
+
+		tfi.consume("cippo", s);
+		gen_ensure(tfi.hasTag("lippo"));
+		gen_ensure(tfi.hasTag("zippo"));
+		gen_ensure(tfi.hasTag("rippo"));
+		s = tfi.getTags("cippo");
+		gen_ensure(s.contains("lippo"));
+		gen_ensure(s.contains("zippo"));
+		gen_ensure(s.contains("rippo"));
+		gen_ensure(!s.contains("bippo"));
+
+		s = tfi.getItems("lippo");
+		gen_ensure(s.contains("cippo"));
+		gen_ensure(!s.contains("lippo"));
+
+		gen_ensure(tfi.getItems("") == Tagcoll::OpSet<string>());
+	} catch (Exception& e) {
+		fprintf(stderr, "%s: %.*s\n", e.type(), PFSTR(e.desc()));
+		throw;
+	}
+
+	// Reopen the database to see if data is actually persisting
+	try {
+		Tagcoll::TDBDiskIndex<string, string> tfi(
+				"pkgidx.test", "tagidx.test",
+				conv, conv, conv, conv);
+		Tagcoll::OpSet<string> s = tfi.getTags("cippo");
+
+		// Repeat the same queries: they should work
+		gen_ensure(s.contains("lippo"));
+		gen_ensure(s.contains("zippo"));
+		gen_ensure(s.contains("rippo"));
+		gen_ensure(! s.contains("bippo"));
+
+		s = tfi.getItems("lippo");
+		gen_ensure(s.contains("cippo"));
+		gen_ensure(!s.contains("lippo"));
+
+		gen_ensure(tfi.getItems("") == Tagcoll::OpSet<string>());
+	} catch (Exception& e) {
+		fprintf(stderr, "%s: %.*s\n", e.type(), PFSTR(e.desc()));
+		throw;
+	}
+}
+
 }
 
 #endif
