@@ -314,10 +314,10 @@ class PatchBuilder : public Consumer<string, string>
 {
 protected:
 	PatchList<ITEM, TAG> patch;
-	Converter<std::string, ITEM> itemconv;
-	Converter<std::string, TAG> tagconv;
+	const Converter<std::string, ITEM>& itemconv;
+	const Converter<std::string, TAG>& tagconv;
 
-	virtual void consumeItemUntagged(const string& item) {}
+	virtual void consumeItemUntagged(const string&) {}
 
 	virtual void consumeItem(const string& item, const OpSet<string>& tags)
 	{
@@ -338,7 +338,7 @@ protected:
 		patch.addPatch(p);
 	}
 
-	virtual void consumeItemsUntagged(const OpSet<string>& items) {}
+	virtual void consumeItemsUntagged(const OpSet<string>&) {}
 
 	virtual void consumeItems(const OpSet<string>& items, const OpSet<string>& tags)
 	{
@@ -418,11 +418,11 @@ void to::test<1>()
 	
 	TestConsumer<string, string> cons;
 
-	Converter<string, string> a;
+	TrivialConverter<string, string> a;
 	TextFormat<string, string>::parse(a, a, coll, cons);
 
-	ensure_equals(cons.items, 4);
-	ensure_equals(cons.tags, 5);
+	gen_ensure_equals(cons.items, 4);
+	gen_ensure_equals(cons.tags, 5);
 }
 
 template<> template<>
@@ -435,7 +435,7 @@ void to::test<2>()
 			"d:  +c::D, -e::F,    -f::g\n"
 	);
 	
-	Converter<string, string> a;
+	TrivialConverter<string, string> a;
 	PatchList<string, string> plist(TextFormat<string, string>::parsePatch(a, a, coll));
 
 	/*
@@ -447,19 +447,19 @@ void to::test<2>()
 			i->second.getRemoved().size()  << endl;
 	*/
 
-	ensure_equals(plist.size(), 2);
+	gen_ensure_equals(plist.size(), 2u);
 	gen_ensure(plist.find("a") != plist.end());
 	gen_ensure(plist.find("b") == plist.end());
 	gen_ensure(plist.find("c") == plist.end());
 	gen_ensure(plist.find("d") != plist.end());
 
 	PatchList<string, string>::const_iterator i = plist.find("a");
-	ensure_equals(i->second.getAdded().size(), 2);
-	ensure_equals(i->second.getRemoved().size(), 0);
+	gen_ensure_equals(i->second.getAdded().size(), 2u);
+	gen_ensure_equals(i->second.getRemoved().size(), 0u);
 
 	i = plist.find("d");
-	ensure_equals(i->second.getAdded().size(), 1);
-	ensure_equals(i->second.getRemoved().size(), 2);
+	gen_ensure_equals(i->second.getAdded().size(), 1u);
+	gen_ensure_equals(i->second.getRemoved().size(), 2u);
 }
 
 }
